import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.MSKClusterEncryption import check


class TestMSKClusterEncryption(unittest.TestCase):

    def test_failure(self):
        resource_conf = {
            "name": "test-project",
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_non_tls(self):
        resource_conf = {
            "name": "test-project",
            "encryption_info": [
                {
                    "encryption_at_rest_kms_key_arn": "aws_kms_key.kms.arn",
                    "encryption_in_transit": [
                        {
                            "client_broker": ["PLAINTEXT"],
                            "in_cluster": ["true"],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_in_cluster(self):
        resource_conf = {
            "name": "test-project",
            "encryption_info": [
                {
                    "encryption_at_rest_kms_key_arn": ["aws_kms_key.kms.arn"],
                    "encryption_in_transit": [
                        {
                            "client_broker": ["TLS"],
                            "in_cluster": [False],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {
            "name": "test-project",
            "encryption_info": [
                {
                    "encryption_at_rest_kms_key_arn": ["aws_kms_key.kms.arn"],
                    "encryption_in_transit": [
                        {
                            "client_broker": ["TLS"],
                            "in_cluster": ["true"],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_no_encrypt_block(self):
        resource_conf = {
            "name": "test-project",
            "encryption_info": [
                {
                    "encryption_at_rest_kms_key_arn": ["aws_kms_key.kms.arn"],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    # Regression test for https://github.com/bridgecrewio/checkov/issues/747
    def test_success_no_encryption_at_rest_kms_key_arn_specified(self):
        resource_conf = {
            "name": "test-project",
            "encryption_info": [{}],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    # Regression test for https://github.com/bridgecrewio/checkov/issues/747
    def test_success_encryption_in_transit_and_no_encryption_at_rest_kms_key_arn_specified(self):
        resource_conf = {
            "name": "test-project",
            "encryption_info": [
                {
                    "encryption_in_transit": [
                        {
                            "client_broker": ["TLS"],
                            "in_cluster": [True],
                        }
                    ],
                }
            ],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.secrets.runner import Runner


class TestCoordinator(unittest.TestCase):

    def test_same_resources_in_report_and_coordinator(self):
        test_root_folder = f'{Path(__file__).parent}'

        secret_runner = Runner()
        report = secret_runner.run(
            root_folder=test_root_folder, runner_filter=RunnerFilter(framework=['secrets'])
        )
        secrets_resources_in_coordinator = set(secret_runner.secrets_coordinator.get_resources())
        failed_resources_from_report = set(f"{record.file_path}:{record.resource}" for record in report.failed_checks)
        self.assertEqual(secrets_resources_in_coordinator, failed_resources_from_report)



import unittest
from pathlib import Path

from checkov.common.models.enums import CheckResult
from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.kubernetes.DockerSocketVolume import check
from checkov.terraform.runner import Runner


class TestDockerSocketVolume(unittest.TestCase):
    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_DockerSocketVolume"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "kubernetes_pod.pass",
            "kubernetes_deployment.pass",
            "kubernetes_daemonset.pass",
            "kubernetes_pod_v1.pass",
            "kubernetes_deployment_v1.pass",
            "kubernetes_daemon_set_v1.pass",
        }

        failing_resources = {
            "kubernetes_pod.fail",
            "kubernetes_deployment.fail",
            "kubernetes_daemonset.fail",
            "kubernetes_pod_v1.fail",
            "kubernetes_deployment_v1.fail",
            "kubernetes_daemon_set_v1.fail",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], 3 * 2)
        self.assertEqual(summary["failed"], 3 * 2)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_terraform_plan(self):
        resource_conf = {'id': ['my_id'], 'metadata': [], 'spec': [{'template': [{'metadata': [], 'spec': [{'volume': [[]]}]}]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        assert scan_result == CheckResult.PASSED


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_eks as eks

class MyEKSClusterStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define EKS Cluster with Encryption Configuration
        cluster = eks.CfnCluster(
            self, 'MyEKSCluster',
            name='my-eks-cluster',
            encryption_config=[{
                'resources': ['secrets']
            }]
            # Other properties for your EKS Cluster
        )

app = core.App()
MyEKSClusterStack(app, "MyEKSClusterStack")
app.synth()



import os
from pathlib import Path

from checkov.github_actions.runner import Runner

RESOURCE_DIR = Path(__file__).parent / "resources/.github/workflows"


def test_schema_validation(bad_schema_files):
    runner = Runner()
    results = {}
    filenames = set()
    for filename in os.listdir(RESOURCE_DIR):
        f = os.path.join(RESOURCE_DIR, filename)
        filenames.add(filename)
        if os.path.isfile(f):
            res = runner._parse_file(f)
            results[filename] = True if res else False

    assert all(results[filename] is True for filename in filenames - bad_schema_files)
    assert all(results[filename] is False for filename in bad_schema_files)


def test_off_value_parsed_correctly():
    # given
    test_file = RESOURCE_DIR / "off_value.yaml"

    # when
    definition, _ = Runner()._parse_file(str(test_file))

    # then
    assert definition["jobs"]["pre-commit"]["steps"][1]["with"]["comment_mode"] is False



from aws_cdk import core
from aws_cdk import aws_backup as backup

class MyBackupStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Backup Vault with the specified encryption key ARN
        backup_vault = backup.CfnBackupVault(
            self,
            "MyBackupVault",
            name="MyBackupVault",
        )



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.RDSClusterEncrypted import check
from checkov.common.models.enums import CheckResult


class TestRDSClusterEncrypted(unittest.TestCase):
    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "aws_rds_global_cluster" "example" {
              provider = aws.primary
            
              global_cluster_identifier = "example"
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_global_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_explicit(self):
        hcl_res = hcl2.loads("""
            resource "aws_rds_global_cluster" "example" {
              provider = aws.primary
            
              global_cluster_identifier = "example"
              storage_encrypted         = false
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_global_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "aws_rds_global_cluster" "example" {
              provider = aws.primary
            
              global_cluster_identifier = "example"
              storage_encrypted         = true
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_global_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_with_source_db_cluster_identifier(self):
        hcl_res = hcl2.loads("""
            resource "aws_rds_global_cluster" "example" {
              provider = aws.primary
            
              global_cluster_identifier = "example"
              source_db_cluster_identifier = "some_arn"
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_global_cluster']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.aws.LambdaDLQConfigured import check
from checkov.common.models.enums import CheckResult


class TestLambdaDLQConfigured(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                        resource "aws_lambda_function" "test_lambda" {
                          filename      = "lambda_function_payload.zip"
                          function_name = "lambda_function_name"
                          role          = aws_iam_role.iam_for_lambda.arn
                          handler       = "exports.test"
                        
                          source_code_hash = filebase64sha256("lambda_function_payload.zip")
                        
                          runtime = "nodejs12.x"
                        
                          environment {
                            variables = {
                              foo = "bar"
                            }
                          }
                        }
                        """)
        resource_conf = hcl_res['resource'][0]['aws_lambda_function']['test_lambda']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                      resource "aws_lambda_function" "test_lambda" {
                          filename      = "lambda_function_payload.zip"
                          function_name = "lambda_function_name"
                          role          = aws_iam_role.iam_for_lambda.arn
                          handler       = "exports.test"
                        
                          source_code_hash = filebase64sha256("lambda_function_payload.zip")
                        
                          runtime = "nodejs12.x"
                          
                          dead_letter_config {
                            target_arn = "test"
                          }
                        
                          environment {
                            variables = {
                              foo = "bar"
                            }
                          }
                        }
                        """)
        resource_conf = hcl_res['resource'][0]['aws_lambda_function']['test_lambda']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from operator import itemgetter
from pathlib import Path

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.models.enums import CheckResult
from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.gitlab_sast import GitLabSast
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.sca.output import create_report_cve_record, _add_to_report_licenses_statuses
from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner


def test_iac_output():
    # given
    test_file = Path(__file__).parent / "fixtures/main.tf"
    report = Runner().run(
        root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV2_AWS_6", "CKV_AWS_18"])
    )

    # when
    gitlab_sast = GitLabSast(reports=[report])
    output = gitlab_sast.sast_json

    # then
    assert (
        output["schema"]
        == "https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/raw/v15.0.4/dist/sast-report-format.json"
    )
    assert output["version"] == "15.0.4"

    # remove dynamic data
    del output["scan"]["start_time"]
    del output["scan"]["end_time"]
    del output["scan"]["analyzer"]["version"]
    assert output["scan"] == {
        "analyzer": {
            "id": "checkov",
            "name": "Checkov",
            "url": "https://www.checkov.io/",
            "vendor": {"name": "Bridgecrew"},
        },
        "scanner": {
            "id": "checkov",
            "name": "Checkov",
            "url": "https://www.checkov.io/",
            "vendor": {"name": "Bridgecrew"},
        },
        "status": "success",
        "type": "sast",
    }

    # remove dynamic data
    for vul in output["vulnerabilities"]:
        del vul["id"]
        del vul["solution"]
        del vul["description"]
        del vul["location"]["file"]
        if "links" in vul:
            del vul["links"]
            for ident in vul["identifiers"]:
                del ident["url"]
    assert sorted(output["vulnerabilities"], key=itemgetter("name")) == sorted(
        [
            {
                "identifiers": [{"name": "CKV2_AWS_6", "type": "checkov", "value": "CKV2_AWS_6"}],
                "location": {"start_line": 1, "end_line": 8},
                "name": "Ensure that S3 bucket has a Public Access block",
                "severity": "Unknown",
            },
            {
                "identifiers": [{"name": "CKV_AWS_18", "type": "checkov", "value": "CKV_AWS_18"}],
                "location": {"start_line": 1, "end_line": 8},
                "name": "Ensure the S3 bucket has access logging enabled",
                "severity": "Unknown",
            },
        ],
        key=itemgetter("name"),
    )


def test_sca_package_output():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package_2.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "moderate",
        "packageName": "django",
        "packageVersion": "1.2",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }

    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses="OSI_BDS",
        package={"package_registry": "https://registry.npmjs.org/", "is_private_registry": False},
    )
    # also add a BC_VUL_2 record
    bc_record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses="OSI_BDS",
        package={"package_registry": "https://registry.npmjs.org/", "is_private_registry": False},
    )
    bc_record.check_id = "BC_VUL_2"

    report = Report(CheckType.SCA_PACKAGE)
    report.add_resource(record.resource)
    report.add_record(record)
    report.add_record(bc_record)

    report.extra_resources.add(
        ExtraResource(
            file_abs_path=file_abs_path,
            file_path=f"/{rootless_file_path}",
            resource=f"{rootless_file_path}.testpkg",
            vulnerability_details={"package_name": "testpkg", "package_version": "1.1.1", "licenses": "MIT"},
        )
    )

    # when
    gitlab_sast = GitLabSast(reports=[report])
    output = gitlab_sast.sast_json

    # then

    # remove dynamic data
    for vul in output["vulnerabilities"]:
        del vul["id"]
    assert output["vulnerabilities"] == [
        {
            "identifiers": [
                {
                    "name": "CVE-2019-19844 - django: 1.2",
                    "type": "cve",
                    "url": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
                    "value": "CVE-2019-19844",
                }
            ],
            "links": [{"url": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844"}],
            "location": {"file": "path/to/requirements.txt"},
            "name": "CVE-2019-19844 - django: 1.2",
            "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
            "severity": "Medium",
            "solution": "fixed in 3.0.1, 2.2.9, 1.11.27",
        },
        {
            "identifiers": [
                {
                    "name": "CVE-2019-19844 - django: 1.2",
                    "type": "cve",
                    "url": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
                    "value": "CVE-2019-19844",
                }
            ],
            "links": [{"url": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844"}],
            "location": {"file": "path/to/requirements.txt"},
            "name": "CVE-2019-19844 - django: 1.2",
            "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
            "severity": "Medium",
            "solution": "fixed in 3.0.1, 2.2.9, 1.11.27",
        },
    ]


def test_sca_license_output():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package_2.scanner.Scanner"
    license_statuses = [
        {
            "package_name": "github.com/apparentlymart/go-textseg/v12",
            "package_version": "v12.0.0",
            "policy": "BC_LIC_1",
            "license": "Apache-2.0",
            "status": "COMPLIANT",
        },
        {
            "package_name": "docutils",
            "package_version": "0.15.2",
            "policy": "BC_LIC_1",
            "license": "GPL",
            "status": "OPEN",
        },
    ]
    sca_details = None

    report = Report(CheckType.SCA_PACKAGE)
    _add_to_report_licenses_statuses(
        report=report,
        check_class=check_class,
        scanned_file_path=file_abs_path,
        rootless_file_path=rootless_file_path,
        runner_filter=RunnerFilter(),
        packages_map=dict(),
        license_statuses=license_statuses,
        sca_details=sca_details,
        report_type=report.check_type,
    )

    # when
    gitlab_sast = GitLabSast(reports=[report])
    output = gitlab_sast.sast_json

    # then

    # remove dynamic data
    for vul in output["vulnerabilities"]:
        del vul["id"]
    assert output["vulnerabilities"] == [
        {
            "identifiers": [{"name": "BC_LIC_1", "type": "license", "value": "BC_LIC_1"}],
            "location": {"file": "path/to/requirements.txt"},
            "name": "License GPL - docutils: 0.15.2",
            "description": "Package docutils@0.15.2 has license GPL",
        }
    ]


def test_create_iac_vulnerability_with_non_url_guideline():
    # given
    # the report doesn't matter for this test, because we pass the record to method directly
    gitlab_sast = GitLabSast(reports=[])
    record = Record(
        check_id="CKV_AWS_5",
        check_name="Ensure all data stored in the Elasticsearch is securely encrypted at rest",
        check_result={"result": CheckResult.FAILED},
        code_block=[],
        file_path="./main.tf",
        file_line_range=[7, 10],
        resource="aws_elasticsearch_domain.enabled",
        evaluations=None,
        check_class="",
        file_abs_path=".",
        bc_check_id="BC_AWS_ELASTICSEARCH_3",
    )
    record.guideline = "Some guideline text"

    # when
    vulnerability = gitlab_sast._create_iac_vulnerability(record=record)

    # then
    # vulnerability["identifiers"][0]["url"] shouldn't exist
    assert vulnerability["identifiers"] == [{"name": "CKV_AWS_5", "type": "checkov", "value": "CKV_AWS_5"}]
    # vulnerability["links"] shouldn't exist
    assert "links" not in vulnerability



import os
import unittest

import hcl2

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.provider.panos.credentials import check
from checkov.common.models.enums import CheckResult
from checkov.terraform.runner import Runner


class TestCredentials(unittest.TestCase):
    def test_success(self):
        hcl_res = hcl2.loads(
            """
            provider "panos" {}
            """
        )
        provider_conf = hcl_res["provider"][0]["panos"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_api(self):
        hcl_res = hcl2.loads(
            """
            provider "panos" {
                api_key = "LUFRPT1yWFdyMFg5NlZxZ1ViU2ZhMTh6aGVEbDJ1UFU9ck9vc2tGcmlHV0tDbWRFa2cxcGUxSU8wMlVjaE9ReU0yYWN5SU1rL2pEOGhDcE50WEt5ABlHQWZoTm8xNG1SQQ=="
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["panos"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_password(self):
        hcl_res = hcl2.loads(
            """
            provider "panos" {
                password = "changeme123!"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["panos"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_api_key(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/resources/api_key"
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources_files = {
            "/pass.tf",
        }

        failing_resources_files = {
            "/fail1.tf",
            "/fail2.tf"
        }

        passed_check_resources_files = set([c.file_path for c in report.passed_checks])
        failing_check_resources_files = set([c.file_path for c in report.failed_checks])

        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["failed"], 2)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources_files, passed_check_resources_files)
        self.assertEqual(failing_resources_files, failing_check_resources_files)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.aws.ECSClusterContainerInsights import check
from checkov.common.models.enums import CheckResult


class TestECSClusterContainerInsights(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads(
            """
            resource "aws_ecs_cluster" "my_cluster" {
                name = "white-hart"
            }
            """
        )
        resource_conf = hcl_res['resource'][0]['aws_ecs_cluster']['my_cluster']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_explicit_disable(self):
        hcl_res = hcl2.loads(
            """
            resource "aws_ecs_cluster" "my_cluster" {
                name = "white-hart"
                setting {
                    name = "containerInsights"
                    value = "disabled"
                }
            }
            """
        )
        resource_conf = hcl_res['resource'][0]['aws_ecs_cluster']['my_cluster']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads(
            """
            resource "aws_ecs_cluster" "my_cluster" {
                name = "white-hart"
                setting {
                    name = "containerInsights"
                    value = "enabled"
                }
            }
            """
        )
        resource_conf = hcl_res['resource'][0]['aws_ecs_cluster']['my_cluster']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_enhanced(self):
        hcl_res = hcl2.loads(
            """
            resource "aws_ecs_cluster" "my_cluster" {
                name = "white-hart"
                setting {
                    name = "containerInsights"
                    value = "enhanced"
                }
            }
            """
        )
        resource_conf = hcl_res['resource'][0]['aws_ecs_cluster']['my_cluster']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == "__main__":
    unittest.main()



from checkov.terraform.checks.data.base_check import BaseDataCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class DummyExternalDataCheck(BaseDataCheck):
    def __init__(self):
        name = "check for terraform data entity"
        id = "CKV_AWS_999"
        supported_resources = ['aws_iam_policy_document']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_data=supported_resources)

    def scan_data_conf(self, conf):
        return CheckResult.PASSED


scanner = DummyExternalDataCheck()



from __future__ import annotations

import pytest

from checkov.common.bridgecrew.bc_source import SourceType
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration, bc_integration


@pytest.fixture()
def mock_bc_integration() -> BcPlatformIntegration:
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.api_url = 'https://www.bridgecrew.cloud'
    bc_integration.setup_bridgecrew_credentials(
        repo_id="bridgecrewio/checkov",
        skip_fixes=True,
        skip_download=True,
        source=SourceType("Github", False),
        source_version="1.0",
        repo_branch="master",
    )
    return bc_integration



from typing import List

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.json_doc.base_json_check import BaseJsonCheck
from checkov.json_doc.enums import BlockType


class FullEvaluatedKey(BaseJsonCheck):
    def __init__(self):
        name = "Ensure that the right configuration block is returned for an evaluated_key that points to an object"
        id = "CKV_RESULT_CONFIG_3"
        categories = [CheckCategories.CONVENTION]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=("*",),
            block_type=BlockType.DOCUMENT,
        )

    def scan_entity_conf(self, conf, entity_type):
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['required_pull_request_reviews/dismissal_restrictions/url']


check = FullEvaluatedKey()



import unittest

from checkov.terraform.checks.resource.aws.APIGatewayAuthorization import check
from checkov.common.models.enums import CheckResult


class TestAPIGatewayAuthorization(unittest.TestCase):

    def test_failure(self):
        resource_conf = {"rest_api_id": ["${var.rest_api_id}"],
                         "resource_id": ["${var.resource_id}"],
                         "http_method": ["${var.method}"],
                         "authorization": ["NONE"]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {"rest_api_id": ["${var.rest_api_id}"],
                         "resource_id": ["${var.resource_id}"],
                         "http_method": ["${var.method}"],
                         "authorization": ["AWS_IAM"]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_apikey(self):
        resource_conf = {"rest_api_id": ["${var.rest_api_id}"],
                         "resource_id": ["${var.resource_id}"],
                         "http_method": ["${var.method}"],
                         "authorization": ["NONE"],
                         "api_key_required": [True]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_authorization_missing(self):
        resource_conf = {"rest_api_id": ["${var.rest_api_id}"],
                         "resource_id": ["${var.resource_id}"],
                         "http_method": ["${var.method}"]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os
import time
from pathlib import Path
from unittest import mock
from unittest.case import TestCase

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_manager import TerraformGraphManager
from checkov.terraform.graph_builder.variable_rendering.renderer import TerraformVariableRenderer
from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
from tests.terraform.graph.variable_rendering.expected_data import (
    expected_terragoat_local_resource_prefix,
    expected_terragoat_db_instance,
    expected_eks,
    expected_provider,
)

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@mock.patch.dict(os.environ, {"RENDER_ASYNC_MAX_WORKERS": "50", "RENDER_VARIABLES_ASYNC": "False"})
class TestRenderer(TestCase):
    def test_render_local(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_local')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_local = {'bucket_name': 'test_bucket_name'}
        expected_resource = {'region': 'us-west-2', 'bucket': expected_local['bucket_name']}

        self.compare_vertex_attributes(local_graph, expected_local, BlockType.LOCALS, 'bucket_name')
        self.compare_vertex_attributes(local_graph, expected_resource, BlockType.RESOURCE, 'aws_s3_bucket.template_bucket')

    def test_render_variable(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_variable')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_resource = {'region': "us-west-2", 'bucket': "test_bucket_name", "acl": "acl", "force_destroy": True}

        self.compare_vertex_attributes(local_graph, expected_resource, BlockType.RESOURCE, 'aws_s3_bucket.template_bucket')

    def test_render_variable_second_resource(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_variable')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_resource = {'region': "us-west-2", 'bucket': "Storage bucket", "acl": "acl", "force_destroy": True}

        self.compare_vertex_attributes(local_graph, expected_resource, BlockType.RESOURCE, 'aws_s3_bucket.storage_bucket')

    def test_render_complex_variable(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/complex_var')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_resource = {'description': 'test', 'name': 'test',
                             'policy': {'Statement': [{'Action': '*',
                                                        'Condition': {'MyCond': {'key': ['0.0.0.0', '1.1.1.1']}},
                                                        'Effect': 'Deny', 'Resource': '*'}], 'Version': '1970-01-01'}}

        self.compare_vertex_attributes(local_graph, expected_resource, BlockType.RESOURCE, 'aws_iam_policy.test')

    def test_render_local_from_variable(self):
        resources_dir = os.path.join(TEST_DIRNAME,
                                     '../resources/variable_rendering/render_local_from_variable')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_local = {'bucket_name': 'test_bucket_name'}

        self.compare_vertex_attributes(local_graph, expected_local, BlockType.LOCALS, 'bucket_name')

    def test_general_example(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/general_example')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_provider = {'profile': 'default', 'region': 'us-east-1', 'alias': 'east1'}
        expected_local = {'bucket_name': {'val': 'MyBucket'}}
        expected_resource = {'region': 'us-west-2', 'bucket': expected_local['bucket_name']}

        self.compare_vertex_attributes(local_graph, expected_provider, BlockType.PROVIDER, 'aws.east1')
        self.compare_vertex_attributes(local_graph, expected_local, BlockType.LOCALS, 'bucket_name')
        self.compare_vertex_attributes(local_graph, expected_resource, BlockType.RESOURCE, 'aws_s3_bucket.template_bucket')

    def test_terragoat_db_app(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_terragoat_db_app')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        self.compare_vertex_attributes(local_graph, expected_terragoat_local_resource_prefix, BlockType.LOCALS, 'resource_prefix')
        self.compare_vertex_attributes(local_graph, expected_terragoat_db_instance, BlockType.RESOURCE, "aws_db_instance.default")

    def test_render_nested_modules(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_nested_modules')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_aws_instance = {"instance_type": "bar"}
        self.compare_vertex_attributes(local_graph, expected_aws_instance, BlockType.RESOURCE, "aws_instance.example")
        expected_output_bucket_acl = {"value": "z"}
        self.compare_vertex_attributes(local_graph, expected_output_bucket_acl, BlockType.OUTPUT, "bucket_acl")

    def compare_vertex_attributes(self, local_graph, expected_attributes, block_type, block_name):
        vertex = local_graph.vertices[local_graph.vertices_block_name_map[block_type][block_name][0]]
        print(f'breadcrumbs = {vertex.breadcrumbs}')
        vertex_attributes = vertex.get_attribute_dict()
        for attribute_key, expected_value in expected_attributes.items():
            actual_value = vertex_attributes.get(attribute_key)
            self.assertEqual(expected_value, actual_value, f'error during comparing {block_type} in attribute key: {attribute_key}')

    def test_breadcrumbs(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/s3_bucket')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)
        vertices = local_graph.vertices
        s3_vertex = list(filter(lambda vertex: vertex.block_type == BlockType.RESOURCE, vertices))[0]
        changed_attributes = list(s3_vertex.changed_attributes.keys())
        self.assertCountEqual(changed_attributes, ['versioning.enabled', 'acl'])

        for breadcrumbs in s3_vertex.changed_attributes.values():
            self.assertEqual(1, len(breadcrumbs))

        acl_origin_vertex = s3_vertex.changed_attributes.get('acl')[0]
        matching_acl_vertex = vertices[acl_origin_vertex.vertex_id]
        self.assertEqual('acl', matching_acl_vertex.name)

        versioning_origin_vertex = s3_vertex.changed_attributes.get('versioning.enabled')[0]
        matching_versioning_vertex = vertices[versioning_origin_vertex.vertex_id]
        self.assertEqual('is_enabled', matching_versioning_vertex.name)

    def test_multiple_breadcrumbs(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/general_example')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)
        vertices = local_graph.vertices
        s3_vertex = list(filter(lambda vertex: vertex.block_type == BlockType.RESOURCE, vertices))[0]
        changed_attributes = list(s3_vertex.changed_attributes.keys())
        self.assertListEqual(changed_attributes, ['region', 'bucket'])

        bucket_vertices_ids_list = s3_vertex.changed_attributes.get('bucket')
        self.assertEqual(2, len(bucket_vertices_ids_list))

        first_vertex = vertices[bucket_vertices_ids_list[0].vertex_id]
        self.assertEqual(BlockType.VARIABLE, first_vertex.block_type)
        self.assertEqual('bucket_name', first_vertex.name)
        self.assertEqual(first_vertex.name, s3_vertex.breadcrumbs['bucket'][0]['name'])

        second_vertex = vertices[bucket_vertices_ids_list[1].vertex_id]
        self.assertEqual(BlockType.LOCALS, second_vertex.block_type)
        self.assertEqual('bucket_name', second_vertex.name)
        self.assertEqual(second_vertex.name, s3_vertex.breadcrumbs['bucket'][1]['name'])

    def test_render_lambda(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_lambda')
        graph_manager = TerraformGraphManager('acme', ['acme'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        expected_aws_lambda_permission = {'count': 0, 'statement_id': 'test_statement_id', 'action': 'lambda:InvokeFunction', 'function_name': 'my-func', 'principal': 'dumbeldor', 'resource_type': 'aws_lambda_permission'}

        self.compare_vertex_attributes(local_graph, expected_aws_lambda_permission, BlockType.RESOURCE, "aws_lambda_permission.test_lambda_permissions")

    def test_eks(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/terraform-aws-eks-master')
        graph_manager = TerraformGraphManager('eks', ['eks'])
        local_graph, tf_def = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        for v in local_graph.vertices:
            expected_v = expected_eks.get(v.block_type, {}).get(v.name)
            if expected_v:
                for attribute_key, expected_value in expected_v.items():
                    actual_value = v.attributes.get(attribute_key)
                    self.assertEqual(expected_value, actual_value,
                                     f'error during comparing {v.block_type} in attribute key: {attribute_key}')


    def test_dict_tfvar(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_dictionary_tfvars')
        graph_manager = TerraformGraphManager('d', ['d'])
        local_graph, tf_def = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        for v in local_graph.vertices:
            expected_v = expected_provider.get(v.block_type, {}).get(v.name)
            if expected_v:
                for attribute_key, expected_value in expected_v.items():
                    actual_value = v.attributes.get(attribute_key)
                    self.assertEqual(expected_value, actual_value,
                                     f'error during comparing {v.block_type} in attribute key: {attribute_key}')

    def test_graph_rendering_order_nested_module_enable(self):
        resource_path = os.path.realpath(os.path.join(TEST_DIRNAME, "..", "resources", "module_rendering", "example"))
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, tf_def = graph_manager.build_graph_from_source_directory(resource_path, render_variables=True)
        module_vertices = list(filter(lambda v: v.block_type == BlockType.MODULE, local_graph.vertices))
        existing = set()
        self.assertEqual(6, len(local_graph.edges))
        for e in local_graph.edges:
            if e in existing:
                self.fail("No 2 edges should be aimed at the same vertex in this example")
            else:
                existing.add(e)
        count = 0
        found = 0
        for v in module_vertices:
            if v.name == 'second-mock':
                found += 1
                if v.attributes['input'] == ['aws_s3_bucket.some-bucket.arn']:
                    count += 1
        self.assertEqual(found, count, f"Expected all instances to have the same value, found {found} instances but only {count} correct values")

    def test_type_default_values(self):
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('map'), {})
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('${map}'), {})
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('map(string)'), {})
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('${map(string)}'), {})
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('list'), [])
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('list(string)'), [])
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('${list}'), [])
        self.assertEqual(TerraformVariableRenderer.get_default_placeholder_value('${list(string)}'), [])
        self.assertIsNone(TerraformVariableRenderer.get_default_placeholder_value('number'))
        self.assertIsNone(TerraformVariableRenderer.get_default_placeholder_value('${number}'))
        self.assertIsNone(TerraformVariableRenderer.get_default_placeholder_value(None))
        self.assertIsNone(TerraformVariableRenderer.get_default_placeholder_value(123))

    def test_tfvar_rendering_module_vars(self):
        resource_path = os.path.join(TEST_DIRNAME, "test_resources", "tfvar_module_variables")
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resource_path, render_variables=True)
        resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
        assert resources_vertex[0].attributes.get('name') == ['airpods']

    def test_dynamic_blocks_with_list(self):
        resource_paths = [
            os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_resource"),
            os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_variable_rendering"),
            os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_tfvars"),

        ]
        for path in resource_paths:
            graph_manager = TerraformGraphManager('m', ['m'])
            local_graph, _ = graph_manager.build_graph_from_source_directory(path, render_variables=True)
            resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
            assert len(resources_vertex[0].attributes.get('ingress')) == 2
            assert len(resources_vertex[0].attributes.get('egress')) == 2
            assert resources_vertex[0].attributes.get('ingress') == \
                   [{'cidr_blocks': ['0.0.0.0/0'], 'from_port': 80, 'protocol': 'tcp', 'to_port': 80},
                    {'cidr_blocks': ['0.0.0.0/0'], 'from_port': 443, 'protocol': 'tcp', 'to_port': 443}]
            assert resources_vertex[0].attributes.get('egress') == \
                   [{'cidr_blocks': ['0.0.0.0/0'], 'from_port': 443, 'protocol': 'tcp', 'to_port': 443},
                    {'cidr_blocks': ['0.0.0.0/0'], 'from_port': 1433, 'protocol': 'tcp', 'to_port': 1433}]

    def test_dynamic_blocks_with_map(self):
        resource_paths = [
            os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_map"),
            os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_map_brackets"),
        ]
        for path in resource_paths:
            graph_manager = TerraformGraphManager('m', ['m'])
            local_graph, _ = graph_manager.build_graph_from_source_directory(path, render_variables=True)
            resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
            assert len(resources_vertex[0].attributes.get('ingress')) == 2
            assert resources_vertex[0].attributes.get('ingress') == \
                   [{'action': 'allow', 'cidr_block': '10.0.0.1/32', 'from_port': 22, 'protocol': 'tcp', 'rule_no': 1,
                     'to_port': 22},
                    {'action': 'allow', 'cidr_block': '10.0.0.2/32', 'from_port': 22, 'protocol': 'tcp', 'rule_no': 2,
                     'to_port': 22}]

    def test_dynamic_blocks_with_nesting_attributes(self):
        root_folder = os.path.join(TEST_DIRNAME, "test_resources", "dynamic_block_nesting_attribute")
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(root_folder, render_variables=True)

        # Test dynamic blocks with nesting attributes
        resource_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))[0]

        assert resource_vertex.attributes.get('server_side_encryption_configuration') == [{'rule': {
            'apply_server_side_encryption_by_default': {'kms_master_key_id': 'testkey1', 'sse_algorithm': 'aws:kms'}}},
            {'rule': {'apply_server_side_encryption_by_default': {'kms_master_key_id': 'testkey2',
                                                                  'sse_algorithm': 'aws:notkms'}}}]

    def test_extract_dynamic_value_in_map(self):
        self.assertEqual(TerraformVariableRenderer.extract_dynamic_value_in_map('value.value1.value2'), 'value2')
        self.assertEqual(TerraformVariableRenderer.extract_dynamic_value_in_map('value.value1["value2"]'), 'value2')

    def test_dynamic_blocks_breadcrumbs(self):
        root_folder = os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_variable_rendering")
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(root_folder, render_variables=True)
        definitions, breadcrumbs = convert_graph_vertices_to_tf_definitions(
            local_graph.vertices,
            root_folder,
        )
        # Test multiple dynamic blocks
        assert 'ingress.from_port' in breadcrumbs['/main.tf']['aws_security_group.list_example']
        assert 'ingress.to_port' in breadcrumbs['/main.tf']['aws_security_group.list_example']
        assert 'egress.to_port' in breadcrumbs['/main.tf']['aws_security_group.list_example']
        assert 'egress.to_port' in breadcrumbs['/main.tf']['aws_security_group.list_example']

        # Test single dynamic block
        assert 'ingress.from_port' in breadcrumbs['/main.tf']['aws_security_group.single_dynamic_example']
        assert 'ingress.to_port' in breadcrumbs['/main.tf']['aws_security_group.single_dynamic_example']

    def test_nested_dynamic_blocks_breadcrumbs(self):
        root_folder = os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_with_nested")
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(root_folder, render_variables=True)
        definitions, breadcrumbs = convert_graph_vertices_to_tf_definitions(
            local_graph.vertices,
            root_folder,
        )
        # Test multiple dynamic blocks
        assert 'required_resource_access.resource_app_id' in breadcrumbs['/main.tf']['azuread_application.bootstrap']
        assert 'required_resource_access.resource_access.id' in breadcrumbs['/main.tf']['azuread_application.bootstrap']
        assert 'required_resource_access.resource_access.type' in breadcrumbs['/main.tf']['azuread_application.bootstrap']

    def test_list_entry_rendering_module_vars(self):
        # given
        resource_path = Path(TEST_DIRNAME) / "test_resources/list_entry_module_var"
        graph_manager = TerraformGraphManager(NetworkxConnector())

        # when
        local_graph, _ = graph_manager.build_graph_from_source_directory(str(resource_path), render_variables=True)

        # then
        resource_vertex = next(v for v in local_graph.vertices if v.id == 'aws_security_group.sg')

        self.assertEqual(
            resource_vertex.config["aws_security_group"]["sg"]["ingress"][0]["cidr_blocks"][0],
            ["0.0.0.0/0"],
        )
        self.assertCountEqual(
            resource_vertex.config["aws_security_group"]["sg"]["egress"][0]["cidr_blocks"][0],
            ["10.0.0.0/16", "0.0.0.0/0"],
        )

        multiple_ingress_vertex = (
            next(v for v in local_graph.vertices if v.id == 'aws_security_group.multiple_ingress_sg'))

        ingress_field = multiple_ingress_vertex.config["aws_security_group"]["multiple_ingress_sg"]["ingress"]
        self.assertEqual(len(ingress_field), 3)

        # TODO - make var rendering correctly evaluate inner vars in list
        self.assertEqual(ingress_field[0],[[]])
        self.assertEqual(ingress_field[1], {'cidr_blocks': ['${var.cidr_sg}'], 'from_port': 23, 'protocol': 'TCP', 'to_port': 23})
        self.assertEqual(ingress_field[2],'var.empty_ingress')

    @mock.patch.dict(os.environ, {"CHECKOV_RENDER_DYNAMIC_MODULES": "False"})
    def test_dynamic_with_env_var_false(self):
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_resource"), render_variables=True)
        resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
        assert not resources_vertex[0].attributes.get('ingress')
        assert not resources_vertex[0].attributes.get('egress')

    def test_dynamic_blocks_with_nested_map(self):
        resource_paths = [
            os.path.join(TEST_DIRNAME, 'test_resources', 'dynamic_blocks_with_nested'),
        ]
        for path in resource_paths:
            graph_manager = TerraformGraphManager('m', ['m'])
            local_graph, _ = graph_manager.build_graph_from_source_directory(path, render_variables=True)
            resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
            assert len(resources_vertex[0].attributes.get('required_resource_access')) == 2
            assert resources_vertex[0].attributes.get('required_resource_access') == \
                   {'resource_app_id': '00000003-0000-0000-c000-000000000000',
                    'resource_access': {'id': '7ab1d382-f21e-4acd-a863-ba3e13f7da61', 'type': 'Role'}}

    def test_dynamic_example_for_security_rule(self):
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(os.path.join(TEST_DIRNAME, "test_resources", "dynamic_block_map_example"), render_variables=True)
        resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
        assert resources_vertex[0].attributes.get('security_rule') == [
            {'access': 'Allow', 'destination_address_prefix': '*', 'destination_port_range': 80, 'direction': 'Inbound', 'name': 'AllowHttpIn', 'priority': 100, 'protocol': 'Tcp', 'source_address_prefix': '*', 'source_port_range': '*'},
            {'access': 'Allow', 'destination_address_prefix': '*', 'destination_port_range': 443, 'direction': 'Inbound', 'name': 'AllowHttpsIn', 'priority': 110, 'protocol': 'Tcp', 'source_address_prefix': '*', 'source_port_range': '*'},
            {'access': 'Allow', 'destination_address_prefix': '*', 'destination_port_range': 3389, 'direction': 'Inbound', 'name': 'AllowRdpIn', 'priority': 120, 'protocol': 'Tcp', 'source_address_prefix': '*', 'source_port_range': '*'},
            {'access': 'Allow', 'destination_address_prefix': '*', 'destination_port_range': '*', 'direction': 'Inbound', 'name': 'AllowIcmpIn', 'priority': 130, 'protocol': 'Icmp', 'source_address_prefix': '*', 'source_port_range': '*'}]
        assert resources_vertex[1].attributes.get('security_rule') == [
            {'access': 'Deny', 'destination_address_prefix': '*', 'destination_port_range': 80, 'direction': 'Inbound', 'name': 'DenyHttpIn', 'priority': 100, 'protocol': 'Tcp', 'source_address_prefix': '*', 'source_port_range': '*'},
            {'access': 'Allow', 'destination_address_prefix': '*', 'destination_port_range': 443, 'direction': 'Inbound', 'name': 'AllowHttpsIn', 'priority': 110, 'protocol': 'Tcp', 'source_address_prefix': '35.181.123.80/32', 'source_port_range': '*'},
            {'access': 'Deny', 'destination_address_prefix': '*', 'destination_port_range': 3389, 'direction': 'Inbound', 'name': 'DenyRdpIn', 'priority': 120, 'protocol': 'Tcp', 'source_address_prefix': '*', 'source_port_range': '*'},
            {'access': 'Deny', 'destination_address_prefix': '*', 'destination_port_range': '*', 'direction': 'Inbound', 'name': 'DenyIcmpIn', 'priority': 130, 'protocol': 'Icmp', 'source_address_prefix': '*', 'source_port_range': '*'}]

    def test_dynamic_blocks_with_nested_lookup(self):
        resource_paths = [
            os.path.join(TEST_DIRNAME, 'test_resources', 'dynamic_nested_with_lookup_foreach'),
        ]
        for path in resource_paths:
            start_time = time.time()
            graph_manager = TerraformGraphManager('m', ['m'])
            local_graph, _ = graph_manager.build_graph_from_source_directory(path, render_variables=True)
            end_time = time.time()
            assert end_time - start_time < 1
            resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE and v.has_dynamic_block, local_graph.vertices))
            assert resources_vertex[0].attributes['stage'] == [
                {'name': 'Source',
                 'action': {'category': 'Source', 'configuration': {'BranchName': 'master', 'PollForSourceChanges': 'false', 'RepositoryName': 'cron-poll'}, 'input_artifacts': [], 'name': 'Source', 'output_artifacts': ['SourceArtifact'], 'owner': 'AWS', 'provider': 'CodeCommit', 'region': '', 'role_arn': 'null', 'run_order': 1, 'version': '1'}},
                {'name': 'Build',
                 'action': {'category': 'Build', 'configuration': {'ProjectName': 'cron-poll'}, 'input_artifacts': ['SourceArtifact'], 'name': 'Build', 'output_artifacts': ['BuildArtifact'], 'owner': 'AWS', 'provider': 'CodeBuild', 'region': '', 'role_arn': 'null', 'run_order': 2, 'version': '1'}},
                {'name': 'Deploy',
                 'action': {'category': 'Deploy', 'configuration': {'ClusterName': 'test', 'ServiceName': 'cron-poll'}, 'input_artifacts': ['BuildArtifact'], 'name': 'Deploy', 'output_artifacts': [], 'owner': 'AWS', 'provider': 'ECS', 'region': '', 'role_arn': 'null', 'run_order': 4, 'version': '1'}}
            ]

    def test_dynamic_blocks_null_lookup(self):
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(
            os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_null_lookup"), render_variables=True)
        resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
        assert len(resources_vertex[0].attributes.get('ingress')) == 2
        assert resources_vertex[0].attributes.get('ingress')[0].get('ipv6_cidr_blocks') == 'null'
        assert resources_vertex[0].attributes.get('ingress')[0].get('self') == 'false'
        assert resources_vertex[0].attributes.get('ingress')[0].get('cidr_blocks') == ['10.248.180.0/23', '10.248.186.0/23']

    def test_dynamic_with_conditional_expression(self):
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(
            os.path.join(TEST_DIRNAME, "test_resources", "dynamic_with_conditional_expression"), render_variables=True)
        resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
        assert resources_vertex[0].attributes.get('identity').get('identity_ids') == 'null'
        assert resources_vertex[0].attributes.get('identity').get('type') == 'SystemAssigned'

    def test_lookup_from_var(self):
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(
            os.path.join(TEST_DIRNAME, "test_resources", "lookup_from_var"), render_variables=True)
        resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
        assert resources_vertex[0].attributes.get('protocol')[0] == 'http'
        assert resources_vertex[0].attributes.get('endpoint')[0] == 'http://www.example.com'

    def test_skip_rendering_unsupported_values(self):
        # given
        resource_path = Path(TEST_DIRNAME) / "test_resources/skip_renderer"

        # when
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(str(resource_path), render_variables=True)

        # then
        local_b = next(vertex for vertex in local_graph.vertices if vertex.block_type == BlockType.LOCALS and vertex.name == "b")
        assert local_b.attributes["b"] == ["..."]  # not Ellipsis object

    def test_default_map_value(self):
        # given
        resource_path = Path(TEST_DIRNAME) / "test_resources/default_map_value"

        # when
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(str(resource_path), render_variables=True)

        # then
        key_vault = next(vertex for vertex in local_graph.vertices if vertex.block_type == BlockType.RESOURCE and vertex.name == "azurerm_key_vault.this")
        assert key_vault.attributes["network_acls"] == {
            "bypass": "AzureServices",
            "default_action": "Deny",
            "ip_rules": [],
            "virtual_network_subnet_ids": []
        }

    def test_provider_alias(self):
        # given
        resource_path = Path(TEST_DIRNAME) / "test_resources/provider_alias"

        # when
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(str(resource_path), render_variables=True)

        # then
        provider = next(vertex for vertex in local_graph.vertices if vertex.block_type == BlockType.PROVIDER and vertex.name == "aws")
        assert provider.config["aws"]["default_tags"] == [{"tags": [{"test": "Test"}]}]

        provider_alias = next(vertex for vertex in local_graph.vertices if vertex.block_type == BlockType.PROVIDER and vertex.name == "aws.test")
        assert provider_alias.config["aws"]["default_tags"] == [{"tags": [{"test": "Test"}]}]

    def test_multiple_dynamic_blocks_value_not_supporting(self):
        resource_paths = [
            os.path.join(TEST_DIRNAME, 'test_resources', 'multiple_dynamic_blocks'),
        ]
        for path in resource_paths:
            graph_manager = TerraformGraphManager('m', ['m'])
            local_graph, _ = graph_manager.build_graph_from_source_directory(path, render_variables=True)

            resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE and v.has_dynamic_block, local_graph.vertices))
            value_block_1 = resources_vertex[0].config['google_sql_database_instance']['instance4-should-fail']['settings'][0]['ip_configuration'][0]['dynamic'][0]['authorized_networks']['content'][0]['value']
            value_block_2 = resources_vertex[0].config['google_sql_database_instance']['instance4-should-fail']['settings'][0]['ip_configuration'][0][
                'dynamic'][1]['authorized_networks']['content'][0]['value']
            # TODO - for now we don't support multiple dynamic blocks - the value_block_1 and value_block_2 needs to be different and not override each other
            assert not value_block_1 != value_block_2


    def test_foreach_with_tfvars(self):
        # given
        resources_dir = Path(TEST_DIRNAME) / "resources/foreach_examples/foreach_tfvars"
        graph_manager = TerraformGraphManager("m", ["m"])

        # when
        local_graph, _ = graph_manager.build_graph_from_source_directory(str(resources_dir), render_variables=True)

        # then
        resource = local_graph.vertices[local_graph.vertices_by_block_type["resource"][0]]
        self.assertDictEqual(
            resource.config["google_project_iam_binding"]['role["roles/run.developer"]'],
            {
                "__address__": 'google_project_iam_binding.role["roles/run.developer"]',
                "__end_line__": 19,
                "__start_line__": 11,
                "members": [["user:captain.america@marvel.com"]],
                "project": ["avengers"],  # this is important it is correctly rendered
                "role": ["roles/run.developer"],
            },
        )

    def test_foreach_with_tfvars_tag_merge(self):
        resource_path = os.path.join(TEST_DIRNAME, "test_resources", "dynamic_blocks_tfvars_merge")
        graph_manager = TerraformGraphManager('m', ['m'])
        local_graph, _ = graph_manager.build_graph_from_source_directory(resource_path, render_variables=True)
        resources_vertex = list(filter(lambda v: v.block_type == BlockType.RESOURCE, local_graph.vertices))
        self.assertDictEqual(resources_vertex[0].config['aws_instance']['this["vm1"]'].get('tags')[0],
                             {'Environment': 'prod', 'Department': 'Testing', 'Name': 'vm1'})



from aws_cdk import core
from aws_cdk import aws_elasticsearch as elasticsearch

class ElasticsearchStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an Amazon Elasticsearch domain
        elasticsearch_domain = elasticsearch.CfnDomain(
            self, "MyElasticsearchDomain",
            domain_name="my-elasticsearch-domain",
            elasticsearch_version="7.10",
            node_to_node_encryption_options={
                "enabled": True
            },
            ebs_options={
                "ebsEnabled": True,
                "volumeSize": 10
            },
        )

# Create the CDK app and stack
app = core.App()
ElasticsearchStack(app, "ElasticsearchStack")
app.synth()



import os
import unittest
from pathlib import Path

from checkov.common.parsers.node import DictNode
from checkov.bicep.parser import Parser
from checkov.bicep.utils import get_scannable_file_paths
from checkov.bicep.graph_builder.context_definitions import build_definitions_context, DEFINITIONS_KEYS_TO_PARSE

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))
RELATIVE_PATH = "resources"
FILE1_NAME = "definitions_example1.bicep"
FILE2_NAME = "definitions_example2.bicep"


class TestDefinitionsContext(unittest.TestCase):
    def setUp(self):
        self.test_root_dir = Path(TEST_DIRNAME) / RELATIVE_PATH
        bicep_parser = Parser()
        self.files_path = get_scannable_file_paths(self.test_root_dir)
        definitions, definitions_raw, parsing_errors = bicep_parser.get_files_definitions(file_paths=self.files_path)
        self.definitions_context = build_definitions_context(definitions, definitions_raw)
        self.file1 = self.definitions_context[f"{self.test_root_dir}/{FILE1_NAME}"]
        self.file2 = self.definitions_context[f"{self.test_root_dir}/{FILE2_NAME}"]

    def test_parameters_values(self):
        self.assertEqual(len(self.definitions_context), len(self.files_path))

        file1_parameters = self.file1[DEFINITIONS_KEYS_TO_PARSE["parameters"]]
        self.assertEqual(len(file1_parameters), 5)
        self.validate_definition_lines(file1_parameters["adminUsername"], 4, 5, 2)
        assert file1_parameters["adminUsername"]["type"] == "string"
        self.validate_definition_lines(file1_parameters["storageAccountType"], 11, 16, 6)
        assert file1_parameters["storageAccountType"]["type"] == "string"
        self.validate_definition_lines(file1_parameters["virtualMachineSize"], 1, 2, 2)
        assert file1_parameters["virtualMachineSize"]["type"] == "string"

        file2_parameters = self.file2[DEFINITIONS_KEYS_TO_PARSE["parameters"]]
        self.assertEqual(len(file2_parameters), 9)
        self.validate_definition_lines(file2_parameters["virtualMachineName"], 1, 1, 1)
        assert file2_parameters["virtualMachineName"]["type"] == "string"
        self.validate_definition_lines(file2_parameters["acrName"], 5, 5, 1)
        assert file2_parameters["acrName"]["type"] == "string"
        self.validate_definition_lines(file2_parameters["publicKey4"], 24, 29, 6)
        assert file2_parameters["publicKey4"]["type"] == "object"

    def test_resources_value(self):
        file1_resources = self.file1[DEFINITIONS_KEYS_TO_PARSE["resources"]]
        self.assertEqual(len(file1_resources), 8)
        self.validate_definition_lines(file1_resources["Microsoft.Compute/virtualMachines.vm"], 33, 82, 50)
        self.validate_definition_lines(file1_resources["Microsoft.Storage/storageAccounts.diagsAccount"], 84, 93, 10)
        self.validate_definition_lines(file1_resources["Microsoft.Network/virtualNetworks.vnet"], 102, 129, 28)

        file2_resources = self.file2[DEFINITIONS_KEYS_TO_PARSE["resources"]]
        self.assertEqual(len(file2_resources), 1)
        self.validate_definition_lines(file2_resources["Microsoft.Compute/virtualMachines.vm"], 31, 72, 42)

    def test_skipped_check_exists(self):
        skipped_checks = self.file1[DEFINITIONS_KEYS_TO_PARSE["resources"]]["Microsoft.Storage/storageAccounts.diagsAccount"]["skipped_checks"]
        self.assertCountEqual(
            skipped_checks,
            {"CKV_AZURE_35":
                 {"result": "SKIPPED", "suppress_comment": " just skip it"},
             "CKV_AZURE_36":
                 {"result": "SKIPPED", "suppress_comment": " skip that too"}
             }
        )

    def validate_definition_lines(self, definition: DictNode, start_line, end_line, code_lines):
        self.assertEqual(definition["start_line"], start_line)
        self.assertEqual(definition["end_line"], end_line)
        self.assertEqual(len(definition["code_lines"]), code_lines)



import unittest
import os
import hcl2

from checkov.terraform.checks.resource.gcp.GoogleComputeBlockProjectSSH import check
from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner
from checkov.common.models.enums import CheckResult


class TestGoogleComputeBlockProjectSSH(unittest.TestCase):

    def test(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/test_GoogleComputeBlockProjectSSH"
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            'google_compute_instance.success1',
            'google_compute_instance.success2',
            'google_compute_instance.success3',
            'google_compute_instance.success4',
            'google_compute_instance_template.success1',
            'google_compute_instance_template.success2',
            'google_compute_instance_template.success3',
            'google_compute_instance_template.success4',
            'google_compute_instance_from_template.success1',
            'google_compute_instance_from_template.success2',
            'google_compute_instance_from_template.success3',
            'google_compute_instance_from_template.success4',
        }
        failing_resources = {
            'google_compute_instance.fail1',
            'google_compute_instance.fail2',
            'google_compute_instance_template.fail1',
            'google_compute_instance_template.fail2',
            'google_compute_instance_from_template.fail1',
        }
        unknown_resources = {
            'google_compute_instance_from_template.unknown1',
            'google_compute_instance_from_template.unknown2',
        }

        passed_check_resources = set([c.resource for c in report.passed_checks])
        failed_check_resources = set([c.resource for c in report.failed_checks])
        #unknown_check_resources = set([c.resource for c in report.failed_checks])

        self.assertEqual(summary['passed'], 12)
        self.assertEqual(summary['failed'], 5)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_unknown_1(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
              metadata = {
                foo = "bar"
                hey = "oh"
                }
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)

    def test_unknown_2(self):
        hcl_res = hcl2.loads("""
            resource "google_compute_instance_from_template" "default" {
              name         = "test"
              source_instance_template = google_compute_instance_template.default.id
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_compute_instance_from_template']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.UNKNOWN, scan_result)

if __name__ == '__main__':
    unittest.main()


from aws_cdk import core
from aws_cdk import aws_iam as iam

class IAMStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an IAM policy
        custom_policy = iam.Policy(
            self,
            "CustomPolicy",
            policy_name="MyCustomPolicy",
            statements=[
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=["s3:GetObject"],
                    resources=["arn:aws:s3:::my-bucket/*"],
                ),
            ],
            users=["a"]
        )


app = core.App()
IAMStack(app, "IAMStack")
app.synth()



from lark import Token

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class S3PCIPrivateACL(BaseResourceCheck):
    def __init__(self):
        name = "Ensure PCI Scope buckets has private ACL (enable public ACL for non-pci buckets)"
        id = "CKV_AWS_999"
        supported_resources = ['aws_s3_bucket']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for ACL configuration at aws_s3_bucket and Tag values:
            https://www.terraform.io/docs/providers/aws/r/s3_bucket.html
        :param conf: aws_s3_bucket configuration
        :return: <CheckResult>
        """
        if 'tags' in conf.keys():
            environment_tag = Token("IDENTIFIER", "Scope")
            if environment_tag in conf['tags'][0].keys():
                if conf['tags'][0][environment_tag] == "PCI":
                    if 'acl' in conf.keys():
                        acl_block = conf['acl']
                        if acl_block in [["public-read"], ["public-read-write"], ["website"]]:
                            return CheckResult.FAILED
        return CheckResult.PASSED


scanner = S3PCIPrivateACL()



from aws_cdk import core
from aws_cdk import aws_kinesis as kinesis

class KinesisStreamStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an Amazon Kinesis stream
        kinesis_stream = kinesis.CfnStream(
            self,
            "MyKinesisStream",
            name="my-kinesis-stream",
            shard_count=2,  # The number of shards in the stream
            stream_encryption={
                "encryption_type": "ABC",
                "key_id": "YOUR_KMS_KEY_ID"  # Replace with your KMS key ID
            }
        )

app = core.App()
KinesisStreamStack(app, "KinesisStreamStack")
app.synth()



from checkov.common.output.report import Report


def test_from_reduced_json(json_reduced_report):
    # Act
    report = Report.from_reduced_json(json_reduced_report, 'mock_check_type')

    # Assert
    assert len(report.failed_checks) == 1
    assert len(report.passed_checks) == 1

    failed_check = report.failed_checks[0]
    passed_check = report.passed_checks[0]

    assert failed_check.check_id == 'CKV_GHA_2'
    assert failed_check.check_name == 'Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables'
    assert failed_check.check_result == {
        "result": "FAILED",
        "results_configuration": {}
    }
    assert failed_check.resource == 'jobs(container-test-job)'
    assert failed_check.file_path == '/.github/workflows/image_no_violation.yml'
    assert failed_check.file_line_range == [7, 7]
    assert failed_check.file_abs_path == '/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml'
    assert failed_check.code_block == [
        [
            7,
            "    runs-on: ubuntu-latest\n"
        ],
    ]
    assert failed_check.bc_check_id == 'BC_REPO_GITHUB_ACTION_1'

    assert passed_check.check_id == 'CKV_GHA_1'
    assert passed_check.check_name == 'Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables'
    assert passed_check.check_result == {
        "result": "PASSED",
        "results_configuration": {}
    }
    assert passed_check.resource == 'jobs(container-test-job)'
    assert passed_check.file_path == '/.github/workflows/image_no_violation.yml'
    assert passed_check.file_line_range == [7, 7]
    assert passed_check.file_abs_path == '/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml'
    assert passed_check.code_block == [
        [
            7,
            "    runs-on: ubuntu-latest\n"
        ],
    ]
    assert passed_check.bc_check_id == 'BC_REPO_GITHUB_ACTION_1'


def test_get_plan_resource_raw_id_1():
    resource_id = Report.get_plan_resource_raw_id("module.vnet[0].azurerm_subnet.subnet_for_each['snet-commonservices']")
    assert resource_id == 'azurerm_subnet.subnet_for_each'


def test_get_plan_resource_raw_id_2():
    resource_id = Report.get_plan_resource_raw_id("module.vnet[0].azurerm_subnet.subnet_for_each[1]")
    assert resource_id == 'azurerm_subnet.subnet_for_each'


def test_get_plan_resource_raw_id_3():
    resource_id = Report.get_plan_resource_raw_id("module.vnet[0].azurerm_subnet.subnet_for_each")
    assert resource_id == 'azurerm_subnet.subnet_for_each'


def test_get_plan_resource_raw_id_4():
    resource_id = Report.get_plan_resource_raw_id("module.vnet.azurerm_subnet.subnet_for_each")
    assert resource_id == 'azurerm_subnet.subnet_for_each'


def test_get_plan_resource_raw_id_5():
    resource_id = Report.get_plan_resource_raw_id("aws_route53_zone.example[\"example.com\"]")
    assert resource_id == 'aws_route53_zone.example'


def test_get_plan_resource_raw_id_6():
    resource_id = Report.get_plan_resource_raw_id("module.sg[\"bad_example\"].aws_security_group.bad")
    assert resource_id == 'aws_security_group.bad'


def test_get_plan_resource_raw_id_7():
    resource_id = Report.get_plan_resource_raw_id("type.name")
    assert resource_id == 'type.name'



import os
from checkov.common.sca.reachability.package_alias_mapping.alias_mapping_creator import AliasMappingCreator
from checkov.common.sca.reachability.package_alias_mapping.nodejs.utils import load_json_with_comments

current_dir = os.path.dirname(os.path.realpath(__file__))


def test_alias_mapping_creator():
    alias_mapping_creator = AliasMappingCreator()
    alias_mapping_creator.update_alias_mapping_for_repository("example_repo", os.path.join(current_dir, "example_repo"), {'axios'})
    alias_mapping = alias_mapping_creator.get_alias_mapping()
    assert alias_mapping == {
        "languages": {
            "nodejs": {
                "repositories": {
                    "example_repo": {
                        "files": {
                            "tsconfig.json": {
                                "packageAliases": {
                                    "axios": {
                                        "packageAliases":["ax"]
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

def test_load_json_with_no_comments():
    json_data_with_comments = """
    {
      "compilerOptions": {
        "paths": {
          "@modules/*": ["src/modules/*"],
          "@shared/*": ["src/shared/*"]
        },
        "declaration": true,
        "target": "es2021",
        "strict": true /* Enable all strict type-checking options. */,
        "noUnusedLocals": false, // off for convenience, enable to enforce cleaner code
        "noUnusedParameters": false, // off for convenience, enable to enforce cleaner code
        "noImplicitAny": false,  // off for convenience, recommended value is true to enforce types and reduce bugs
        "forceConsistentCasingInFileNames": true /* Disallow inconsistently-cased references to the same file. */,
        "resolveJsonModule": true
      },
      "exclude": ["node_modules", "dist"]
    }
    """
    clean_json = load_json_with_comments(json_data_with_comments)
    assert clean_json == {
        "compilerOptions": {
            "paths": {
                "@modules/*": ["src/modules/*"],
                "@shared/*": ["src/shared/*"]
            },
            "declaration": True,
            "target": "es2021",
            "strict": True,
            "noUnusedLocals": False,
            "noUnusedParameters": False,
            "noImplicitAny": False,
            "forceConsistentCasingInFileNames": True,
            "resolveJsonModule": True
        },
        "exclude": ["node_modules", "dist"]
    }


import os
import unittest

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.code_categories import CodeCategoryType, CodeCategoryMapping
from checkov.common.bridgecrew.integration_features.features.licensing_integration import LicensingIntegration
from checkov.common.bridgecrew.licensing import CustomerSubscription, SubscriptionCategoryMapping, \
    CategoryToSubscriptionMapping, open_source_categories
from checkov.common.bridgecrew.platform_errors import ModuleNotEnabledError
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.runner_filter import RunnerFilter

from checkov.bitbucket.runner import Runner as bitbucket_configuration_runner
from checkov.sca_package_2.runner import Runner as sca_package_runner_2
from checkov.secrets.runner import Runner as secrets_runner
from checkov.terraform.runner import Runner as tf_graph_runner

# limited set for shorter testing
DEFAULT_RUNNERS = (
    tf_graph_runner(),
    secrets_runner(),
    bitbucket_configuration_runner(),
    sca_package_runner_2()
)

checkov_runners = [value for attr, value in CheckType.__dict__.items() if not attr.startswith("__")]

# pycharm gives false positive "unresolved reference" - ignore https://youtrack.jetbrains.com/issue/PY-36205
module_keys = [e.value for e in CustomerSubscription]

runner_to_subscription_map = {runner: CategoryToSubscriptionMapping.get(CodeCategoryMapping[runner]) for runner in checkov_runners if 'sca_' not in runner}
runner_to_subscription_map['sca_package'] = CustomerSubscription.SCA
runner_to_subscription_map['sca_image'] = CustomerSubscription.SCA
subscription_to_runner_map = {CustomerSubscription(sub): [runner for runner in checkov_runners if runner_to_subscription_map.get(runner) == CustomerSubscription(sub)] for sub in module_keys}


class TestLicensingIntegration(unittest.TestCase):

    def test_constants(self):
        # these tests ensure that these lists get maintained if the runners and categories change
        self.assertEqual(set(module_keys), {'IAC', 'SECRETS', 'SCA', 'SAST'})

        self.assertEqual(set(checkov_runners), {
            'ansible',
            'argo_workflows',
            'arm',
            'azure_pipelines',
            'bicep',
            'bitbucket_configuration',
            'bitbucket_pipelines',
            'cdk',
            'circleci_pipelines',
            'cloudformation',
            'dockerfile',
            'github_configuration',
            'github_actions',
            'gitlab_configuration',
            'gitlab_ci',
            'helm',
            'json',
            'kubernetes',
            'kustomize',
            'openapi',
            'sca_package',
            'sca_image',
            'secrets',
            'serverless',
            'terraform',
            'terraform_json',
            'terraform_plan',
            'yaml',
            'sast',
            'sast_python',
            'sast_java',
            'sast_javascript',
            'sast_typescript',
            'sast_golang',
            '3d_policy'
        })

        self.assertEqual(SubscriptionCategoryMapping.get(CustomerSubscription.IAC), (CodeCategoryType.IAC, CodeCategoryType.BUILD_INTEGRITY))
        self.assertEqual(SubscriptionCategoryMapping.get(CustomerSubscription.SCA), (CodeCategoryType.LICENSES, CodeCategoryType.VULNERABILITIES))
        self.assertEqual(SubscriptionCategoryMapping.get(CustomerSubscription.SECRETS), (CodeCategoryType.SECRETS,))
        self.assertEqual(SubscriptionCategoryMapping.get(CustomerSubscription.SAST), (CodeCategoryType.WEAKNESSES,))

        self.assertEqual(CategoryToSubscriptionMapping[CodeCategoryType.IAC], CustomerSubscription.IAC)
        self.assertEqual(CategoryToSubscriptionMapping[CodeCategoryType.BUILD_INTEGRITY], CustomerSubscription.IAC)
        self.assertEqual(CategoryToSubscriptionMapping[CodeCategoryType.LICENSES], CustomerSubscription.SCA)
        self.assertEqual(CategoryToSubscriptionMapping[CodeCategoryType.VULNERABILITIES], CustomerSubscription.SCA)
        self.assertEqual(CategoryToSubscriptionMapping[CodeCategoryType.SECRETS], CustomerSubscription.SECRETS)
        self.assertEqual(CategoryToSubscriptionMapping[CodeCategoryType.WEAKNESSES], CustomerSubscription.SAST)

        self.assertEqual(CodeCategoryMapping.get(CheckType.BITBUCKET_PIPELINES), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.CIRCLECI_PIPELINES), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.ANSIBLE), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.ARM), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.AZURE_PIPELINES), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.BICEP), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.CDK), CodeCategoryType.WEAKNESSES)
        self.assertEqual(CodeCategoryMapping.get(CheckType.CLOUDFORMATION), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.DOCKERFILE), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.GITHUB_CONFIGURATION), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.GITHUB_ACTIONS), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.GITLAB_CONFIGURATION), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.GITLAB_CI), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.BITBUCKET_CONFIGURATION), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.HELM), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.JSON), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.YAML), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.KUBERNETES), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.KUSTOMIZE), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.OPENAPI), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.SCA_PACKAGE), [CodeCategoryType.LICENSES, CodeCategoryType.VULNERABILITIES])
        self.assertEqual(CodeCategoryMapping.get(CheckType.SCA_IMAGE), [CodeCategoryType.LICENSES, CodeCategoryType.VULNERABILITIES])
        self.assertEqual(CodeCategoryMapping.get(CheckType.SECRETS), CodeCategoryType.SECRETS)
        self.assertEqual(CodeCategoryMapping.get(CheckType.SERVERLESS), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.TERRAFORM), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.TERRAFORM_PLAN), CodeCategoryType.IAC)
        self.assertEqual(CodeCategoryMapping.get(CheckType.ARGO_WORKFLOWS), CodeCategoryType.BUILD_INTEGRITY)
        self.assertEqual(CodeCategoryMapping.get(CheckType.SAST), CodeCategoryType.WEAKNESSES)

        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.BITBUCKET_PIPELINES), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.CIRCLECI_PIPELINES), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.ANSIBLE), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.ARM), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.AZURE_PIPELINES), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.BICEP), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.CDK), CustomerSubscription.SAST)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.CLOUDFORMATION), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.DOCKERFILE), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.GITHUB_CONFIGURATION), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.GITHUB_ACTIONS), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.GITLAB_CONFIGURATION), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.GITLAB_CI), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.BITBUCKET_CONFIGURATION), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.HELM), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.JSON), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.YAML), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.KUBERNETES), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.KUSTOMIZE), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.OPENAPI), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.SCA_PACKAGE), CustomerSubscription.SCA)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.SCA_IMAGE), CustomerSubscription.SCA)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.SECRETS), CustomerSubscription.SECRETS)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.SERVERLESS), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.TERRAFORM), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.TERRAFORM_PLAN), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.ARGO_WORKFLOWS), CustomerSubscription.IAC)
        self.assertEqual(LicensingIntegration.get_subscription_for_runner(CheckType.SAST), CustomerSubscription.SAST)

        self.assertEqual(open_source_categories, [CodeCategoryType.IAC, CodeCategoryType.SECRETS, CodeCategoryType.BUILD_INTEGRITY])

    def test_integration_valid(self):
        instance = BcPlatformIntegration()
        instance.skip_download = False
        instance.platform_integration_configured = True

        # it is always valid, because it always makes a determination for which runners run
        licensing_integration = LicensingIntegration(instance)

        self.assertTrue(licensing_integration.is_valid())

        instance.skip_download = True
        self.assertTrue(licensing_integration.is_valid())

        instance.platform_integration_configured = False
        self.assertTrue(licensing_integration.is_valid())

        instance.skip_download = False
        self.assertTrue(licensing_integration.is_valid())

        licensing_integration.integration_feature_failures = True
        self.assertTrue(licensing_integration.is_valid())

    def test_oss_mode_enabled(self):
        instance = BcPlatformIntegration()

        licensing_integration = LicensingIntegration(instance)

        licensing_integration.pre_scan()
        self.assertTrue(licensing_integration.open_source_only)  # no API key

        instance.bc_api_key = '1234'
        licensing_integration.pre_scan()
        self.assertTrue(licensing_integration.open_source_only)  # no customer run config

        # IAC and secrets are valid, SCA is not
        for runner_check_type in checkov_runners:
            self.assertEqual(
                licensing_integration.is_runner_valid(runner_check_type),
                runner_to_subscription_map[runner_check_type] not in (CustomerSubscription.SCA, CustomerSubscription.SAST),
            )

    def test_oss_mode_resource_plan(self):
        instance = BcPlatformIntegration()
        licensing_integration = LicensingIntegration(instance)
        instance.bc_api_key = '1234'
        licensing_integration.pre_scan()

        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {m: True for m in module_keys},
            }
        }
        licensing_integration.pre_scan()
        self.assertFalse(licensing_integration.open_source_only)

    def test_oss_mode_dev_plan(self):
        instance = BcPlatformIntegration()
        licensing_integration = LicensingIntegration(instance)
        instance.bc_api_key = '1234'
        licensing_integration.pre_scan()

        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {m: True for m in module_keys},
            }
        }
        licensing_integration.pre_scan()
        self.assertFalse(licensing_integration.open_source_only)

    def test_resource_mode(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'

        licensing_integration = LicensingIntegration(instance)

        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {m: True for m in module_keys},
            }
        }

        licensing_integration.pre_scan()

        for runner_check_type in checkov_runners:
            self.assertTrue(licensing_integration.is_runner_valid(runner_check_type))
        self.assertTrue(licensing_integration.should_run_image_referencer())

    def test_developer_mode_all_enabled(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'

        licensing_integration = LicensingIntegration(instance)

        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {key: True for key in module_keys},
            }
        }

        licensing_integration.pre_scan()

        for runner_check_type in checkov_runners:
            if runner_check_type.startswith("sast"):  # todo: remove when sast will be active
                continue
            self.assertTrue(licensing_integration.is_runner_valid(runner_check_type))
        self.assertTrue(licensing_integration.should_run_image_referencer())

    def test_developer_mode_all_disabled(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'

        licensing_integration = LicensingIntegration(instance)

        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {key: False for key in module_keys},
            }
        }

        licensing_integration.pre_scan()

        for runner_check_type in checkov_runners:
            if runner_check_type.startswith(("cdk", "sast")):  # todo: remove when sast will be active
                continue
            self.assertFalse(licensing_integration.is_runner_valid(runner_check_type))
        self.assertFalse(licensing_integration.should_run_image_referencer())

    def test_developer_mode_each_enabled(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'

        licensing_integration = LicensingIntegration(instance)

        # test one module at a time
        for module in module_keys:
            instance.customer_run_config_response = {
                'platformLicense': {
                    'modules': {key: key == module for key in module_keys},
                }
            }
            licensing_integration.pre_scan()
            for runner_check_type in checkov_runners:
                if runner_check_type.startswith(("cdk", "sast")):  # todo: remove when sast will be active
                    continue
                self.assertEqual(licensing_integration.is_runner_valid(runner_check_type), runner_check_type in subscription_to_runner_map[CustomerSubscription(module)])
            self.assertEqual(licensing_integration.should_run_image_referencer(), module == 'SCA')

    def test_runner_registry_single_runner(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'
        licensing_integration = LicensingIntegration(instance)
        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {m: True for m in module_keys},
            }
        }

        licensing_integration.pre_scan()

        scan_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources')

        runner_filter = RunnerFilter(framework=['terraform'], runners=checkov_runners)
        runner_registry = RunnerRegistry('', runner_filter, *DEFAULT_RUNNERS)
        runner_registry.licensing_integration = licensing_integration
        reports = runner_registry.run(root_folder=scan_dir)
        self.assertEqual(len(reports), 1)
        self.assertIsNotNone(reports[0])

    def test_runner_registry_single_runner_hard_fail(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'
        licensing_integration = LicensingIntegration(instance)
        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {m: False for m in module_keys},
            }
        }

        licensing_integration.pre_scan()

        scan_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources')

        runner_filter = RunnerFilter(framework=['terraform'], runners=checkov_runners)
        runner_registry = RunnerRegistry('', runner_filter, *DEFAULT_RUNNERS)
        runner_registry.licensing_integration = licensing_integration
        try:
            runner_registry.run(root_folder=scan_dir)
            raise AssertionError('Runner registry should hard fail because a single framework was used')
        except Exception as e:
            self.assertIsInstance(e, ModuleNotEnabledError)

    def test_runner_registry_multiple_runners_with_framework(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'
        licensing_integration = LicensingIntegration(instance)
        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {
                    'IAC': True,
                    'SECRETS': False,
                    'SCA': False
                },
            }
        }

        licensing_integration.pre_scan()

        scan_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources')

        runner_filter = RunnerFilter(framework=['terraform', 'bitbucket_configuration', 'sca_package', 'secrets'], runners=checkov_runners)
        runner_registry = RunnerRegistry('', runner_filter, *DEFAULT_RUNNERS)
        runner_registry.licensing_integration = licensing_integration
        with self.assertLogs(level='INFO') as log:
            reports = runner_registry.run(root_folder=scan_dir)
            self.assertEqual(len(reports), 2)  # terraform and bitbucket
            # we are specifically verifying the log level here
            self.assertIn('WARNING:root:The framework "secrets" is part of the "SECRETS" module, which is not enabled in the platform', log.output)
            self.assertIn('WARNING:root:The framework "secrets" is part of the "SECRETS" module, which is not enabled in the platform', log.output)

    def test_runner_registry_multiple_runners_without_framework(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'
        licensing_integration = LicensingIntegration(instance)
        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {
                    'IAC': True,
                    'SECRETS': False,
                    'SCA': False
                },
            }
        }

        licensing_integration.pre_scan()

        scan_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources')

        runner_filter = RunnerFilter(runners=checkov_runners)
        runner_registry = RunnerRegistry('', runner_filter, *DEFAULT_RUNNERS)
        runner_registry.licensing_integration = licensing_integration
        with self.assertLogs(level='INFO') as log:
            reports = runner_registry.run(root_folder=scan_dir)
            self.assertEqual(len(reports), 2)  # terraform and bitbucket
            # we are specifically verifying the log level here
            self.assertIn('INFO:root:The framework "secrets" is part of the "SECRETS" module, which is not enabled in the platform', log.output)
            self.assertIn('INFO:root:The framework "secrets" is part of the "SECRETS" module, which is not enabled in the platform', log.output)

    def test_runner_registry_multiple_runners_all_disabled(self):
        instance = BcPlatformIntegration()
        instance.bc_api_key = '1234'
        licensing_integration = LicensingIntegration(instance)
        instance.customer_run_config_response = {
            'platformLicense': {
                'modules': {
                    'IAC': False,
                    'SECRETS': False,
                    'SCA': False
                }
            }
        }

        licensing_integration.pre_scan()

        scan_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources')

        runner_filter = RunnerFilter(framework=['terraform', 'bitbucket_configuration', 'sca_package', 'secrets'], runners=checkov_runners)
        runner_registry = RunnerRegistry('', runner_filter, *DEFAULT_RUNNERS)
        runner_registry.licensing_integration = licensing_integration
        try:
            runner_registry.run(root_folder=scan_dir)
            raise AssertionError('Runner registry should hard fail because a single framework was used')
        except Exception as e:
            self.assertIsInstance(e, ModuleNotEnabledError)


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_efs as efs

class EfsStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an EFS file system with the Encrypted property set to True
        efs_file_system = efs.FileSystem(
            self,
            "EfsFileSystem",
            encrypted=True,  # Set Encrypted property to True
            lifecycle_policy=efs.LifecyclePolicy.AFTER_7_DAYS,
        )

app = core.App()
EfsStack(app, "EfsStack")
app.synth()



from aws_cdk import core
from aws_cdk import aws_logs as logs

class MyLogGroupStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a CloudWatch Logs log group with KMS key ID
        log_group = logs.LogGroup(
            self,
            "MyLogGroup",
            log_group_name="MyLogGroupName",
            retention=logs.RetentionDays.ONE_MONTH,  # Set the retention policy as needed
            kms_key=1,  # Specify the KMS key
        )



from checkov.terraform.context_parsers.base_parser import BaseContextParser


class MockContextParser(BaseContextParser):
    def __init__(self):
        definition_type = 'mock'
        self.definition_type = definition_type

    def enrich_definition_block(self, definition_blocks):
        """
        Enrich the context of a Terraform resource block
        :param definition_blocks: Terraform resource block, key-value dictionary
        :return: Enriched resource block context
        """
        parsed_file_lines = self._filter_file_lines()

        for i, mock_block in enumerate(definition_blocks):
            mock_type = next(iter(mock_block.keys()))
            mock_name = next(iter(mock_block[mock_type]))
        if not self.context.get(mock_type):
            self.context[mock_type] = {}
        if not self.context.get(mock_type).get(mock_name):
            self.context[mock_type][mock_name] = {}
        for line_num, line in parsed_file_lines:
            line_tokens = [x.replace('"', "") for x in line.split()]
            if all(x in line_tokens for x in ['mock', mock_type, mock_name]):
                self.context[mock_type][mock_name]["start_line"] = 1
                self.context[mock_type][mock_name]["end_line"] = 5
                self.context[mock_type][mock_name]["code_lines"] = ['ABC', '123']
        return self.context

    def get_block_type(self):
        return "resource"

    def get_entity_context_path(self, entity_block):
        entity_type = next(iter(entity_block.keys()))
        entity_name = next(iter(entity_block[entity_type]))
        return [entity_type, entity_name]



from aws_cdk import core
from aws_cdk import aws_ec2 as ec2

class MyVolumeStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an EBS volume with encryption enabled
        ebs_volume = ec2.Volume(
            self,
            "MyEBSVolume",
            availability_zone="us-east-1a",  # Replace with your desired availability zone
            size=100,  # Set the size of the volume as needed
            encrypted=True,  # Enable encryption
            volume_type=ec2.EbsDeviceVolumeType.GP2,  # Specify the volume type
        )



import os
from typing import Dict, Any
from checkov.common.sca.reachability.package_alias_mapping.nodejs.nodejs_alias_mapping_strategy import NodejsAliasMappingStrategy

current_dir = os.path.dirname(os.path.realpath(__file__))


def test_create_alias_mapping_from_webpack_file():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "webpack")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'webpack.config.js': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_babelrc_file():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "babel", "babelrc")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'.babelrc': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_babel_config_file():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "babel", "babel_config")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'babel.config.js': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_rollup_file():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "rollup")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'rollup.config.js': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_package_json_alias():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "package_json", "package_json_with_alias")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'package.json': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_package_json_aliasify():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "package_json", "package_json_with_aliasify")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'package.json': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_snowpack():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "snowpack")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'snowpack.config.js': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_vite():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "vite")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'vite.config.js': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_mix():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "mix")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {'nodejs': {'repositories': {'supplygoat': {'files': {'vite.config.js': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}, 'package_json_with_alias/package.json': {'packageAliases': {'axios': {'packageAliases': ['ax']}}}}}}}}}


def test_create_alias_mapping_from_fake():
    strategy_object = NodejsAliasMappingStrategy()
    root_dir = os.path.join(current_dir, "examples", "fake_file")
    alias_mapping: Dict[str, Any] = {"languages": {}}
    strategy_object.update_alias_mapping(alias_mapping, "supplygoat", root_dir, {'axios'})
    assert alias_mapping == {'languages': {}}



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.IAMStarActionPolicyDocument import check


class TestIAMStarActionPolicyDocument(unittest.TestCase):
    def test_success(self):
        resource_conf = {
            "name": ["test"],
            "user": ["${aws_iam_user.lb.name}"],
            "policy": [
                '{"Version": "2012-10-17", "Statement": [{"Action": ["ec2:Describe*"], "Effect": "Allow", "Resource": "abc*"}]}'
            ],
        }
        scan_result = check.scan_entity_conf(conf=resource_conf, entity_type="aws_iam_policy")
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_service_star(self):
        resource_conf = {
            "name": ["test"],
            "user": ["${aws_iam_user.lb.name}"],
            "policy": [
                {"Version": "2012-10-17", "Statement": [{"Action": "ec2:*", "Effect": "Allow", "Resource": "abc*"}]}
            ],
        }
        scan_result = check.scan_entity_conf(conf=resource_conf, entity_type="aws_iam_policy")
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        resource_conf = {
            "name": ["test"],
            "user": ["${aws_iam_user.lb.name}"],
            "policy": [
                {"Version": "2012-10-17", "Statement": [{"Action": ["*"], "Effect": "Allow", "Resource": "abc*"}]}
            ],
        }
        scan_result = check.scan_entity_conf(conf=resource_conf, entity_type="aws_iam_policy")
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_multiple_statements(self):
        resource_conf = {
            "name": ["test"],
            "user": ["${aws_iam_user.lb.name}"],
            "policy": [
                {
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Sid": "SqsAllow",
                            "Effect": "Allow",
                            "Action": [
                                "sqs:GetQueueAttributes",
                                "sqs:GetQueueUrl",
                                "sqs:ListDeadLetterSourceQueues",
                                "sqs:ListQueues",
                                "sqs:ReceiveMessage",
                                "sqs:SendMessage",
                                "sqs:SendMessageBatch",
                            ],
                            "Resource": "*",
                        },
                        {"Sid": "ALL", "Effect": "Allow", "Action": ["*"], "Resource": ["${var.my_resource_arn}"]},
                    ],
                }
            ],
        }
        scan_result = check.scan_entity_conf(conf=resource_conf, entity_type="aws_iam_policy")
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_sso_failure(self):

        resource_conf = {
            "instance_arn": "${aws_ssoadmin_permission_set.example.arn}",
            "permission_set_arn": "${aws_iam_user.lb.name}",
            "inline_policy": [
                {"Version": "2012-10-17", "Statement": [{"Action": ["*"], "Effect": "Allow", "Resource": "abc*"}]}
            ],
        }
        scan_result = check.scan_entity_conf(
            conf=resource_conf, entity_type="aws_ssoadmin_permission_set_inline_policy"
        )
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_sso_success(self):
        resource_conf = {
            "instance_arn": "${aws_ssoadmin_permission_set.example.arn}",
            "permission_set_arn": "${aws_iam_user.lb.name}",
            "inline_policy": [
                '{"Version": "2012-10-17", "Statement": [{"Action": ["ec2:Describe*"], "Effect": "Allow", "Resource": "abc*"}]}'
            ],
        }
        scan_result = check.scan_entity_conf(
            conf=resource_conf, entity_type="aws_ssoadmin_permission_set_inline_policy"
        )
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == "__main__":
    unittest.main()



from pathlib import Path

from checkov.argo_workflows.runner import Runner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.images.image_referencer import Image
from checkov.runner_filter import RunnerFilter
from checkov.argo_workflows.checks.registry import registry

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_registry_has_type():
    assert registry.report_type == CheckType.ARGO_WORKFLOWS


def test_runner_honors_enforcement_rules():
    # given
    test_file = EXAMPLES_DIR / "hello_world.yaml"

    # when
    filter = RunnerFilter(framework=['argo_workflows'], use_enforcement_rules=True)
    # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
    # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
    filter.enforcement_rule_configs = {CheckType.ARGO_WORKFLOWS: Severities[BcSeverities.OFF]}
    report = Runner().run(root_folder="", files=[str(test_file)], runner_filter=filter)

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


def test_runner_passing_check():
    # given
    test_file = EXAMPLES_DIR / "hello_world.yaml"

    # when
    report = Runner().run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_ARGO_2"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 1
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


def test_runner_failing_check():
    # given
    test_file = EXAMPLES_DIR / "hello_world.yaml"

    # when
    report = Runner().run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter(checks=["CKV_ARGO_1"]))

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 1
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0


def test_runner_ignore_argo_cd():
    # given
    test_file = EXAMPLES_DIR / "argo_cd_application.yaml"

    # when
    report = Runner().run(root_folder="", files=[str(test_file)], runner_filter=RunnerFilter())

    # then
    summary = report.get_summary()

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0
    assert summary["resource_count"] == 0


def test_get_image():
    # given
    test_file = EXAMPLES_DIR / "scripts_python.yaml"

    # when
    images = Runner().get_images(str(test_file))

    # then
    assert images == {
        Image(
            file_path=str(test_file),
            name="alpine:latest",
            start_line=33,
            end_line=36,
        ),
        Image(
            file_path=str(test_file),
            name="python:alpine3.6",
            start_line=22,
            end_line=28,
        ),
    }




import os
from unittest import mock

import pytest
from checkov.common.images.image_referencer import Image
from checkov.terraform.image_referencer.provider.aws import AwsTerraformProvider
from tests.graph_utils.utils import GRAPH_FRAMEWORKS, set_graph_by_graph_framework, \
    add_vertices_to_graph_by_graph_framework


@mock.patch.dict(os.environ, {"BC_ROOT_DIR": "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src"})
@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_extract_images_from_resources_with_external_module(graph_framework):
    # given
    resource = {
        "file_path_": "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/aws_batch_job_definition.batch.tf",
        "__end_line__": 8,
        "__start_line__": 1,
        "container_definitions": [
            {
                "name": "first",
                "image": "nginx",
                "cpu": 10,
                "memory": 512,
                "essential": True,
                "portMappings": [{"containerPort": 80, "hostPort": 80}],
            },
            {
                "name": "second",
                "image": "python:3.9-alpine",
                "cpu": 10,
                "memory": 256,
                "essential": True,
                "portMappings": [{"containerPort": 443, "hostPort": 443}],
            },
        ],
        "resource_type": "aws_ecs_task_definition",
        "module_dependency_": "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf",
        "module_dependency_num_": "0",
        "id_": "aws_batch_job_definition.batch",
    }
    module_resource = {
        "block_name_": "batch",
        "block_type_": "module",
        "file_path_": "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf",
        "config_": {
            "batch": {
                "__end_line__": 21,
                "__resolved__": [
                    "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/aws_batch_compute_environment.batch.tf[/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf#0]",
                    "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/aws_batch_job_definition.batch.tf[/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf#0]",
                    "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/aws_batch_job_queue.batch.tf[/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf#0]",
                    "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/aws_batch_scheduling_policy.pike.tf[/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf#0]",
                    "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/outputs.tf[/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf#0]",
                    "/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/variables.tf[/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/example/examplea/module.batch.tf#0]"
                ],
                "__start_line__": 1,
            }
        },
        "id": "5c440d2a1a5c656290cdf8f98e1d893b1c08f7d7bb7cb93ff97a1884b83c18cc"
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)
    add_vertices_to_graph_by_graph_framework(graph_framework, module_resource, graph, 2, 'batch', 'module')

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        aws_provider = AwsTerraformProvider(graph_connector=graph)
        images = aws_provider.extract_images_from_resources()

    # then
    assert images == [
        Image(
            file_path='/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/aws_batch_job_definition.batch.tf',
            name="nginx",
            start_line=1,
            end_line=8,
            related_resource_id='/aws_batch_job_definition.batch.tf:module.batch.aws_batch_job_definition.batch'
        ),
        Image(
            file_path='/tmp/checkov/cshayner/cshayner/terraform-aws-batch/master/src/aws_batch_job_definition.batch.tf',
            name="python:3.9-alpine",
            start_line=1,
            end_line=8,
            related_resource_id='/aws_batch_job_definition.batch.tf:module.batch.aws_batch_job_definition.batch'
        ),
    ]


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_extract_images_from_resources(graph_framework):
    # given
    resource = {
        "file_path_": "/ecs.tf",
        "__end_line__": 31,
        "__start_line__": 1,
        "container_definitions": [
            {
                "name": "first",
                "image": "nginx",
                "cpu": 10,
                "memory": 512,
                "essential": True,
                "portMappings": [{"containerPort": 80, "hostPort": 80}],
            },
            {
                "name": "second",
                "image": "python:3.9-alpine",
                "cpu": 10,
                "memory": 256,
                "essential": True,
                "portMappings": [{"containerPort": 443, "hostPort": 443}],
            },
        ],
        "resource_type": "aws_ecs_task_definition",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        aws_provider = AwsTerraformProvider(graph_connector=graph)
        images = aws_provider.extract_images_from_resources()

    # then
    assert images == [
        Image(
            file_path="/ecs.tf",
            name="nginx",
            start_line=1,
            end_line=31,
            related_resource_id='/ecs.tf:None'
        ),
        Image(
            file_path="/ecs.tf",
            name="python:3.9-alpine",
            start_line=1,
            end_line=31,
            related_resource_id='/ecs.tf:None'
        ),
    ]


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_extract_images_from_resources_with_no_image(graph_framework):
    # given
    resource = {
        "file_path_": "/ecs.tf",
        "__end_line__": 31,
        "__start_line__": 1,
        "container_definitions": [
            {
                "name": "first",
                "cpu": 10,
                "memory": 512,
                "essential": True,
                "portMappings": [{"containerPort": 80, "hostPort": 80}],
            },
        ],
        "resource_type": "aws_ecs_task_definition",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        aws_provider = AwsTerraformProvider(graph_connector=graph)
        images = aws_provider.extract_images_from_resources()

    # then
    assert not images



from typing import Dict, List, Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.data.base_check import BaseDataCheck
from checkov.common.util.oidc_utils import gh_abusable_claims, gh_repo_regex, gh_sub_condition


class GithubActionsOIDCTrustPolicy(BaseDataCheck):
    def __init__(self):
        name = "Ensure AWS GitHub Actions OIDC authorization policies only allow safe claims and claim order"
        id = "CKV_AWS_358"
        supported_data = ("aws_iam_policy_document",)
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_data=supported_data)

    def scan_data_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        statements = force_list(conf.get("statement"))
        for statement in statements:
            found_federated_gh_oidc = False
            if isinstance(statement, dict):
                if statement.get("principals"):
                    principals = statement["principals"]
                    for principal in force_list(principals):
                        if "type" not in principal and "identifiers" not in principal:
                            continue
                        principal_type = principal["type"]
                        principal_identifiers = principal["identifiers"]
                        if (
                            isinstance(principal_type, list)
                            and len(principal_type)
                            and "Federated" in principal_type
                            and isinstance(principal_identifiers, list)
                        ):
                            for identifier in principal_identifiers:
                                if (
                                    isinstance(identifier, list)
                                    and len(identifier) > 0
                                    and identifier[0] is not None
                                    and "oidc-provider/token.actions.githubusercontent.com" in identifier[0]
                                ):
                                    found_federated_gh_oidc = True
                                    break
                if not found_federated_gh_oidc:
                    return CheckResult.PASSED

                # By now we know that the statement is a federated GitHub OIDC provider
                # First check - if the statement is a federated GitHub OIDC provider, it MUST have a condition
                if found_federated_gh_oidc and not statement.get("condition"):
                    return CheckResult.FAILED
                found_sub_condition_variable = False
                found_sub_condition_value = False

                # It is common to have multiple conditions, so we need to iterate over them
                for condition in statement.get("condition"):
                    condition_variables = condition.get("variable")
                    condition_values = condition.get("values")
                    if isinstance(condition_variables, list):
                        for condition_variable in condition_variables:
                            if gh_sub_condition.match(condition_variable):
                                found_sub_condition_variable = True
                                break

                        # If we didn't find the sub condition variable, we can skip the rest of the checks
                        if not found_sub_condition_variable:
                            continue
                        if isinstance(condition_values, list):
                            for condition_value in condition_values:
                                if isinstance(condition_value, list):
                                    # First -> check if the value is a mere wildcard. If so, it's a fail
                                    # This covers the case where the condition is ['sub':'*']
                                    if len(condition_value) == 1 and condition_value[0] == "*":
                                        return CheckResult.FAILED
                                    # Split the claims by ':' for deeper inspection
                                    split_claims = condition_value[0].split(":")
                                    # The assertion MUST be of the form ['{claim_name_1}:{claim_value_1}:{claim_name_2}:{claim_value_2}...']
                                    # If the length of the split claims is 1, it means that the assertion is ['sub':'{claim_name}'] - this is a fail
                                    if len(split_claims) == 1:
                                        return CheckResult.FAILED
                                    # Second -> Check if the value is a wildcard assertion
                                    # This covers the case where the condition is ['sub':'{claim_name}:*']
                                    if split_claims[1] == "*":
                                        return CheckResult.FAILED
                                    # Third -> Check if the value is an abusable claim
                                    # This covers the case where the condition is ['sub':'{abusable_claim}:{any_value}']
                                    for abusable_claim in gh_abusable_claims:
                                        if split_claims[0].startswith(abusable_claim):
                                            return CheckResult.FAILED
                                    # Fourth -> Check if the value is a repo:org/* -> this is a pass with a warning
                                    if split_claims[0] == "repo" and not gh_repo_regex.match(split_claims[1]):
                                        return CheckResult.FAILED
                                    found_sub_condition_value = True
                                    break
                        if found_sub_condition_value and found_sub_condition_variable:
                            return CheckResult.PASSED

                # Found a federated GitHub user, but no restrictions
                return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ["statement/condition/variable", "statement/condition/values"]


check = GithubActionsOIDCTrustPolicy()
